// ---------------------------------------------------------------------------
// test_matrix.cpp - Testprogramm fuer die HMMatrix-Funktionsbibliothek
//
// Die einzelnen Funktionen werden mittels "assert" ueberprueft, das
// Testprogramm muss aus diesem Grund im Debug-Modus uebersetzt werden.
// Falls das Testprogramm ohne Debug-Assertion-Fehler ausgefuehrt wird,
// arbeitet die Vektor-/Matrix-Bibliothek korrekt.
//
//               Copyright Tilman Kuepper 2009 - 2017.
//    Distributed under the Boost Software License, Version 1.0.
//        (See accompanying file LICENSE_1_0.txt or copy at
//               http://www.boost.org/LICENSE_1_0.txt)
//
// Letzte Bearbeitung: 1. August 2017
// ---------------------------------------------------------------------------

#include "matrix.hpp"
#include <assert.h>
#include <stdio.h>    // tmpnam(), remove()
#include <iostream>
#include <sstream>
#include <algorithm>  // sort()

// Suche nach Memory-Leaks bei Microsoft Visual C++ aktivieren
#ifdef _MSC_VER
#include <crtdbg.h>
#endif

#ifdef NDEBUG
#error Achtung, test_matrix.cpp muss im Debug-Modus compiliert werden!
#endif

using namespace std;
using namespace HMMatrix;

// ---------------------------------------------------------------------------
// Vergleich zweier Double-Zahlen auf Gleichheit, Rundungsfehler ignorieren
// ---------------------------------------------------------------------------
inline bool is_equal(double x, double y)
{
    const double epsilon = 0.001;
    return (x - y) * (x - y) < epsilon * epsilon;
}

// ---------------------------------------------------------------------------
// Vergleich zweier Matrizen auf Gleichheit, Rundungsfehler ignorieren
// ---------------------------------------------------------------------------
inline bool is_equal(const Matrix& a, const Matrix& b)
{
    const size_t dim = a.Dimension();
    assert(dim == b.Dimension());
    for(size_t row = 0; row < dim; ++row)
        for(size_t col = 0; col < dim; ++col)
            if(!is_equal(a(row, col), b(row, col))) return false;
    return true;
}

// ---------------------------------------------------------------------------
// Initialisierung von bzw. Zugriff auf Vektoren testen
// ---------------------------------------------------------------------------
void Vector1()
{
    cout << "Vector1()...\n";

    Vector v(3);
    v.Elements() = 0., 2., 3.;
    v(0) = v(0) + .5;   
    v[0] = v[0] + .5;   

    assert(is_equal(v.GetMax(), 3.0));
    assert(is_equal(v.GetMin(), 1.0));

    assert(v.Dimension() == 3);
    assert(&v(0) == v.Data());

    assert(is_equal(v(0), 1.));
    assert(is_equal(v(1), 2.));
    assert(is_equal(v(2), 3.));

    assert(is_equal(v[0], 1.));
    assert(is_equal(v[1], 2.));
    assert(is_equal(v[2], 3.));

    assert(v == v);
    assert(!(v != v));

    v.Init(MAT_INIT_NULL);
    assert(is_equal(v.Abs(), 0.));

    // Zufallszahlen zwischen 0 und 1...
    v.Init(MAT_INIT_RANDOM);
    assert(v(0) + v(1) + v(2) >= 0.);
    assert(v(0) + v(1) + v(2) <= 3.);

    // Vektor mit angegebenem Wert fuellen...
    v.Fill(1.5);
    assert(is_equal(v[0] + v[1] + v[2], 4.5));

    // Kopier-Konstruktor...
    Vector v1(v);
    assert(is_equal(v[0], v1[0]));
    assert(is_equal(v[1], v1[1]));
    assert(is_equal(v[2], v1[2]));

    // Zugriff auf Vektor per Iterator...
    Vector v2(100);
    v2.Init(MAT_INIT_RANDOM);
    std::sort(v2.begin(), v2.end());
    Vector::const_iterator it = v2.begin() + 1;
    for(; it != v2.end(); ++it)
        assert(*it >= *(it - 1));

    // Anzahl der Elemente im Vektor bestimmen...
    assert(v2.size() == v2.Dimension());
    assert(v2.size() == (size_t)(v2.end() - v2.begin()));

    // push_back, pop_back, clear, empty...
    Vector push_pop(0);
    push_pop.push_back(0);
    push_pop.push_back(1);
    push_pop.push_back(2);
    assert(push_pop.size() == 3);
    assert(push_pop.empty() == false);
    assert(push_pop(0) == 0);
    assert(push_pop(1) == 1);
    assert(push_pop(2) == 2);
    push_pop.pop_back();
    assert(push_pop.size() == 2);
    push_pop.clear();
    assert(push_pop.size() == 0);
    assert(push_pop.empty() == true);

    // Vector::Slice() testen...
    Vector slice_test(0);
    for(size_t i = 0; i < 100; ++i)
    {
        slice_test.push_back(i);
        slice_test.push_back(i);
    }
    assert(slice_test.Slice(0, 100, 2) == slice_test.Slice(1, 100, 2));
    assert(slice_test.Slice(0, 100, 2).size() == 100);
    assert(slice_test.Slice(0, 3)(0) == 0);
    assert(slice_test.Slice(0, 3)(1) == 0);
    assert(slice_test.Slice(0, 3)(2) == 1);
}

// ---------------------------------------------------------------------------
// Grundrechenarten von Vektoren testen
// ---------------------------------------------------------------------------
void Vector2()
{
    cout << "Vector2()...\n";

    Vector v1(3), v2(3), v3(3), v4(3), v5(3);
    v1.Init(MAT_INIT_RANDOM);
    v2.Init(MAT_INIT_RANDOM);
    v3 = v1 - v2;

    v1 = v1 * 2.;
    v2 = v2 * 2.;
    v4 = (v1 - v2) / -2.0;

    v5 = v3 + v4;
    assert(is_equal(v5.Abs(), 0.));

    v1.Init(MAT_INIT_RANDOM);
    v2 = v1;
    v1 = v1 * 2;
    v2 = 2 * v2;
    assert(is_equal((v1 - v2).Abs(), 0));
}

// ---------------------------------------------------------------------------
// Weitere Funktionalitaeten von Vektoren testen
// ---------------------------------------------------------------------------
void Vector3()
{
    cout << "Vector3()...\n";

    // Normal, Normalize, Abs...
    Vector v1(5);
    v1.Init(MAT_INIT_RANDOM);
    Vector v2(v1);

    v2 = v2.Normal();
    v1.Normalize();

    assert(is_equal(v1.Abs(), 1.));
    assert(is_equal(v1(0), v2(0)));
    assert(is_equal(v1(1), v2(1)));
    assert(is_equal(v1(2), v2(2)));
    assert(is_equal(v1(3), v2(3)));
    assert(is_equal(v1(4), v2(4)));

    // Skalar- und Kreuzprodukt...
    Vector a(3), b(3), c(3);
    a.Elements() = 1, 2, 3;
    b.Elements() = 3, 2, 1;
    assert(is_equal(a * b, 1*3 + 2*2 + 3*1));

    c = CrossProduct(a, b);
    assert(is_equal(c(0), 2*1 - 2*3));
    assert(is_equal(c(1), 3*3 - 1*1));
    assert(is_equal(c(2), 1*2 - 2*3));

    // Skalarprodukt von grossen Vektoren...
    for(size_t big_size = 1000; big_size < (size_t)1010; ++big_size)
    {
        Vector big1(big_size), big2(big_size);
        big1.Init(MAT_INIT_RANDOM);
        big2.Init(MAT_INIT_RANDOM);
        double tmp = big1 * big2;
        for(size_t x = 0; x < big_size; ++x) tmp -= big1[x] * big2[x];
        assert(is_equal(tmp, 0));
    }
}

// ---------------------------------------------------------------------------
// Formatierte Ausgabe von Vektoren auf Streams testen
// ---------------------------------------------------------------------------
void Vector4()
{
    cout << "Vector4()...\n";

    Vector v(2);
    v.Elements() = 1.5, -2.5;

    // Erst einmal eine "normale" Zahl formatiert ausgeben...
    stringstream strm;
    strm.width(8); strm.precision(4);
    strm << 1.2345678;
    assert(strm.str() == "   1.235");

    // Nun einen Vektor im "Standardformat"...
    strm.str("");
    strm << v;
    assert(strm.str() == "    1.50;    -2.50");

    // Nun einen Vektor in benutzerdefiniertem Format...
    strm.str("");
    strm << MatrixFormat(4, 1, true, true, ":") << v;
    assert(strm.str() == " 1.5:-2.5\n");

    // Testen, ob das urspruengliche Format des Streams noch intakt ist...
    strm.str("");
    strm << 1.2345678;
    assert(strm.str() == "1.235");

    // Einlesen von Vektoren im MATLAB-Format...
    Vector v1 = MATLABVector( "[1,2,3]");
    Vector v2 = MATLABVector( "[1 2 3]");
    Vector v3 = MATLABVector( "[1;2;3]");
    Vector v4 = MATLABVector(L"[1;2;3]");
    assert(v1(0) == 1);
    assert(v1(1) == 2);
    assert(v2(2) == 3);
    assert(v1.size() == 3);
    assert(v1 == v2);
    assert(v1 == v3);
    assert(v1 == v4);

    // Bei Einlesefehlern werden Ausnahmen ausgeloest...
    try { MATLABVector("[]"      ); assert(!"a"); } catch(...) { }
    try { MATLABVector("[1 2 3"  ); assert(!"b"); } catch(...) { }
    try { MATLABVector("1 2 3]"  ); assert(!"c"); } catch(...) { }
    try { MATLABVector("[1 2,,3]"); assert(!"d"); } catch(...) { }
}

// ---------------------------------------------------------------------------
// Fast Fourier Transformation
// ---------------------------------------------------------------------------
void Vector5()
{
    cout << "Vector5()...\n";

    // Zum Test wird die FFT eines reinen Gleichanteils berechnet...
    Vector y1(8);
    y1.Fill(1.0);
    DComplexVector Y = y1.FFT();

    assert(is_equal(Y.at(0).real(), 8));  // <--- (!)
    assert(is_equal(Y.at(1).real(), 0));
    assert(is_equal(Y.at(2).real(), 0));
    assert(is_equal(Y.at(3).real(), 0));
    assert(is_equal(Y.at(4).real(), 0));
    assert(is_equal(Y.at(5).real(), 0));
    assert(is_equal(Y.at(6).real(), 0));
    assert(is_equal(Y.at(7).real(), 0));

    assert(is_equal(Y.at(0).imag(), 0));
    assert(is_equal(Y.at(1).imag(), 0));
    assert(is_equal(Y.at(2).imag(), 0));
    assert(is_equal(Y.at(3).imag(), 0));
    assert(is_equal(Y.at(4).imag(), 0));
    assert(is_equal(Y.at(5).imag(), 0));
    assert(is_equal(Y.at(6).imag(), 0));
    assert(is_equal(Y.at(7).imag(), 0));

    // Jetzt die inverse FFT berechnen...
    DComplexVector y2 = IFFT(Y);

    assert(is_equal(y2.at(0).real(), 1));
    assert(is_equal(y2.at(1).real(), 1));
    assert(is_equal(y2.at(2).real(), 1));
    assert(is_equal(y2.at(3).real(), 1));
    assert(is_equal(y2.at(4).real(), 1));
    assert(is_equal(y2.at(5).real(), 1));
    assert(is_equal(y2.at(6).real(), 1));
    assert(is_equal(y2.at(7).real(), 1));

    assert(is_equal(y2.at(0).imag(), 0));
    assert(is_equal(y2.at(1).imag(), 0));
    assert(is_equal(y2.at(2).imag(), 0));
    assert(is_equal(y2.at(3).imag(), 0));
    assert(is_equal(y2.at(4).imag(), 0));
    assert(is_equal(y2.at(5).imag(), 0));
    assert(is_equal(y2.at(6).imag(), 0));
    assert(is_equal(y2.at(7).imag(), 0));

    // Noch ein Test mit einem anderen Vektor...
    Vector z1(4);
    z1.Elements() = 1, 2, 3, 4;
    DComplexVector Z = FFT(z1);

    assert(is_equal(Z.at(0).real(), 10));
    assert(is_equal(Z.at(1).real(), -2));
    assert(is_equal(Z.at(2).real(), -2));
    assert(is_equal(Z.at(3).real(), -2));

    assert(is_equal(Z.at(0).imag(),  0));
    assert(is_equal(Z.at(1).imag(),  2));
    assert(is_equal(Z.at(2).imag(),  0));
    assert(is_equal(Z.at(3).imag(), -2));

    // ...und wieder zurueck...
    DComplexVector z2 = IFFT(Z);

    assert(is_equal(z1[0], z2[0].real()));
    assert(is_equal(z1[1], z2[1].real()));
    assert(is_equal(z1[2], z2[2].real()));
    assert(is_equal(z1[3], z2[3].real()));

    assert(is_equal(0, z2[0].imag()));
    assert(is_equal(0, z2[1].imag()));
    assert(is_equal(0, z2[2].imag()));
    assert(is_equal(0, z2[3].imag()));
}

// ---------------------------------------------------------------------------
// Initialisierung von bzw. Zugriff auf Matrizen testen
// ---------------------------------------------------------------------------
void Matrix1()
{
    cout << "Matrix1()...\n";

    Matrix m(3);
    m.Row(0) = 1, 2, 3;
    m.Row(1) = 2, 3, 4;
    m.Row(2) = 3, 4, 5;

    assert(m.Dimension() == 3);
    assert(m.Data() == &m(0, 0));

    assert(is_equal( 6, m(0, 0) + m(0, 1) + m(0, 2)));
    assert(is_equal( 9, m(1, 0) + m(1, 1) + m(1, 2)));
    assert(is_equal(12, m(2, 0) + m(2, 1) + m(2, 2)));

    assert(m == m);
    assert(!(m != m));

    m.Init(MAT_INIT_NULL);
    assert(is_equal(0, m.GetRow(0).Abs()));
    assert(is_equal(0, m.GetRow(1).Abs()));
    assert(is_equal(0, m.GetRow(2).Abs()));
    assert(is_equal(0, m.GetCol(0).Abs()));
    assert(is_equal(0, m.GetCol(1).Abs()));
    assert(is_equal(0, m.GetCol(2).Abs()));

    m.Init(MAT_INIT_IDENTITY);
    assert(is_equal(1, m.GetRow(0).Abs()));
    assert(is_equal(1, m.GetRow(1).Abs()));
    assert(is_equal(1, m.GetRow(2).Abs()));
    assert(is_equal(1, m.GetCol(0).Abs()));
    assert(is_equal(1, m.GetCol(1).Abs()));
    assert(is_equal(1, m.GetCol(2).Abs()));

    m.Init(MAT_INIT_HILBERT);
    assert(is_equal(1.0 / 1.0, m(0, 0)));
    assert(is_equal(1.0 / 2.0, m(0, 1)));
    assert(is_equal(1.0 / 3.0, m(0, 2)));
    assert(is_equal(1.0 / 2.0, m(1, 0)));
    assert(is_equal(1.0 / 3.0, m(1, 1)));
    assert(is_equal(1.0 / 4.0, m(1, 2)));
    assert(is_equal(1.0 / 3.0, m(2, 0)));
    assert(is_equal(1.0 / 4.0, m(2, 1)));
    assert(is_equal(1.0 / 5.0, m(2, 2)));
    assert(is_equal(m.GetMax(), 1.0 / 1.0));
    assert(is_equal(m.GetMin(), 1.0 / 5.0));

    Matrix i(3);
    i.Init(MAT_INIT_INV_HILBERT);
    m *= i;
    assert(is_equal(1, m(0, 0)));
    assert(is_equal(0, m(0, 1)));
    assert(is_equal(0, m(0, 2)));
    assert(is_equal(0, m(1, 0)));
    assert(is_equal(1, m(1, 1)));
    assert(is_equal(0, m(1, 2)));
    assert(is_equal(0, m(2, 0)));
    assert(is_equal(0, m(2, 1)));
    assert(is_equal(1, m(2, 2)));

    m.Init(MAT_INIT_RANDOM);
    assert(m(0, 0) >= 0. && m(0, 0) <= 1.);
    assert(m(0, 1) >= 0. && m(0, 1) <= 1.);
    assert(m(0, 2) >= 0. && m(0, 2) <= 1.);
    assert(m(1, 0) >= 0. && m(1, 0) <= 1.);
    assert(m(1, 1) >= 0. && m(1, 1) <= 1.);
    assert(m(1, 2) >= 0. && m(1, 2) <= 1.);
    assert(m(2, 0) >= 0. && m(2, 0) <= 1.);
    assert(m(2, 1) >= 0. && m(2, 1) <= 1.);
    assert(m(2, 2) >= 0. && m(2, 2) <= 1.);

    // Matrix mit angegebenem Wert fuellen
    m.Fill(-1.5);
    assert(is_equal(-1.5, m(0, 0)));
    assert(is_equal(-1.5, m(0, 1)));
    assert(is_equal(-1.5, m(0, 2)));
    assert(is_equal(-1.5, m(1, 0)));
    assert(is_equal(-1.5, m(1, 1)));
    assert(is_equal(-1.5, m(1, 2)));
    assert(is_equal(-1.5, m(2, 0)));
    assert(is_equal(-1.5, m(2, 1)));
    assert(is_equal(-1.5, m(2, 2)));

    // Zugriff auf die Hauptdiagonale
    Matrix mat(10);
    mat.Init(MAT_INIT_HILBERT);
    Vector diag = mat.GetDiag();
    for(size_t n = 0; n < mat.Dimension(); n++)
        assert(is_equal(mat(n, n), diag(n)));

    // Zugriff auf Matrix per Iterator
    std::sort(mat.begin(), mat.end());
    assert(is_equal(mat(0, 0), 1.0 / 19.0));
    assert(is_equal(mat(9, 9), 1.0 /  1.0));
    Matrix::const_iterator it = mat.begin() + 1;
    for(; it != mat.end(); ++it)
        assert(*it >= *(it - 1));

    // Anzahl der Elemente in der Matrix bestimmen
    assert(mat.size() == mat.Dimension() * mat.Dimension());
    assert(mat.size() == (size_t)(mat.end() - mat.begin()));

    // Test auf Symmetrie
    mat.Init(MAT_INIT_HILBERT);
    assert(mat.IsSymmetric(0.0001));

    mat(5, 1) += 0.001;
    assert(!mat.IsSymmetric(0.0001));

    // Matrix wieder symmetrisch machen
    mat(4, 0) = 0; mat(0, 4) = 10;
    mat.MakeSymmetric(MAT_USE_MEAN);
    assert(is_equal(mat(4, 0), 5) && is_equal(mat(0, 4), 5));
    assert(mat.IsSymmetric(0.0001));

    mat(4, 0) = 0; mat(0, 4) = 10;
    mat.MakeSymmetric(MAT_USE_UPPER);
    assert(is_equal(mat(4, 0), 10) && is_equal(mat(0, 4), 10));
    assert(mat.IsSymmetric(0.0001));

    mat(4, 0) = 0; mat(0, 4) = 10;
    mat.MakeSymmetric(MAT_USE_LOWER);
    assert(is_equal(mat(4, 0), 0) && is_equal(mat(0, 4), 0));
    assert(mat.IsSymmetric(0.0001));
}

// ---------------------------------------------------------------------------
// Berechnungen mit Matrizen testen
// ---------------------------------------------------------------------------
void Matrix2()
{
    cout << "Matrix2()...\n";

    Matrix a(4), b(4);
    a.Row(0) = 1, 2, 3, 4;
    a.Row(1) = 1, 2, 3, 4;
    a.Row(2) = 1, 2, 3, 4;
    a.Row(3) = 1, 2, 3, 4;

    a = (a * 2) - (a / 2) - (a * 0.5) + (a * (-0.5)) + (a / (-2));
    b.Init(MAT_INIT_NULL);
    assert(is_equal(a, b));

    a.Init(MAT_INIT_IDENTITY);
    b.Init(MAT_INIT_IDENTITY);
    a = a * 2;
    b = 2 * b;
    assert(is_equal(a, b));

    a.Init(MAT_INIT_HILBERT);
    b.Init(MAT_INIT_INV_HILBERT);
    assert(is_equal(a.Inverse(), b));
    a.Invert();
    assert(is_equal(a, b));

    Vector v1(4), v2(4);
    a.Init(MAT_INIT_IDENTITY);
    v1.Init(MAT_INIT_RANDOM);
    v2 = (a * 2) * v1 / 2;
    assert(is_equal((v2 - v1).Abs(), 0));

    // Bei grossen Matrizen wird eine Cache-freundliche Variante der Matrix-
    // Multiplikation durchgefuehrt. Testen, ob das Ergebnis der optimierten
    // Matrix-Multiplikation mit der "naiven" Variante uebereinstimmt.
    for(size_t my_dim = 95; my_dim <= 100; ++my_dim)
    {
        Matrix A(my_dim), B(my_dim);
        A.Init(MAT_INIT_RANDOM);
        B.Init(MAT_INIT_RANDOM);
        Matrix C = A * B;

        // ..."naive" Variante zum Vergleich...
        for(size_t row = 0; row < my_dim; ++row)
        {
            for(size_t col = 0; col < my_dim; ++col)
            {
                double tmp = 0;
                for(size_t idx = 0; idx < my_dim; ++idx)
                    tmp += A(row, idx) * B(idx, col);
                assert(is_equal(C(row, col), tmp));
            }
        }
    }
}

// ---------------------------------------------------------------------------
// Weitere Funktionalitaeten von Matrizen testen
// ---------------------------------------------------------------------------
void Matrix3()
{
    cout << "Matrix3()...\n";

    Matrix a(3), b(3), c(3), q(3), r(3), aa(1), qq(1), rr(1);
    a.Row(0) = 1, 2, 3;
    a.Row(1) = 1, 2, 3;
    a.Row(2) = 9, 8, 7;

    b = a.GetTranspose();
    a.Transpose();

    c.Row(0) = 1, 1, 9;
    c.Row(1) = 2, 2, 8;
    c.Row(2) = 3, 3, 7;

    assert(is_equal(a, c));
    assert(is_equal(b, c));

    a.Transpose();
    assert(is_equal(a.Determinant(), 1*2*7+2*3*9+3*1*8-9*2*3-8*3*1-7*1*2));

    // QR-Zerlegung einer 3-dimensionalen Matrix
    a.Row(0) = 12, -51,   4;
    a.Row(1) =  6, 167, -68;
    a.Row(2) = -4,  24, -41;

    a.QRDecomp(q, r);
    assert(is_equal(a, q * r));

    b.Row(0) = 14,  21, -14;
    b.Row(1) =  0, 175, -70;
    b.Row(2) =  0,   0,  35;
    assert(is_equal(r, b));

    c = a * r.Inverse();
    assert(is_equal(q, c));

    Vector test(3);
    test = a.GetCol(1);
    assert(is_equal(test.Abs() * test.Abs(), 51*51+167*167+24*24));
    test = a.GetRow(1);
    assert(is_equal(test.Abs() * test.Abs(), 6*6+167*167+68*68));

    a.SwapRow(1, 2);
    test = a.GetCol(1);
    assert(is_equal(test.Abs() * test.Abs(), 51*51+167*167+24*24));
    test = a.GetRow(2);
    assert(is_equal(test.Abs() * test.Abs(), 6*6+167*167+68*68));

    // QR-Zerlegung einer 1-dimensionalen Matrix
    aa(0, 0) = 123;
    aa.QRDecomp(qq, rr);
    assert(is_equal(qq(0, 0),   1.0));
    assert(is_equal(rr(0, 0), 123.0));

    // LU-Zerlegung einer 5-dimensionalen Matrix
    Matrix aaa(5), lll(5), uuu(5), ppp(5);
    aaa.Init(MAT_INIT_HILBERT);
    aaa.LUDecomp(lll, uuu, ppp);
    for(size_t row = 0; row < 5; ++row)
    {
        size_t col;
        for(col = 0; col < row; ++col)
            assert(is_equal(uuu(row, col), 0.0));
        assert(is_equal(lll(row, row), 1.0));
        for(col = row + 1; col < 3; ++col)
            assert(is_equal(lll(row, col), 0.0));
    }
    assert(is_equal(aaa, ppp * lll * uuu));

    // LU-Zerlegung, Invertierung von singulaerer Matrix --> Ausnahmefehler!
    aaa.Init(MAT_INIT_NULL);
    try { aaa.Inverse();               assert(!"Exception?"); } catch(...) { }
    try { aaa.LUDecomp(lll, uuu, ppp); assert(!"Exception?"); } catch(...) { }

    // Invertierung einer eindimensionalen Matrix
    aa(0, 0) = 10;
    aa.Invert();
    assert(is_equal(aa(0, 0), 0.1));
}

// ---------------------------------------------------------------------------
// Formatierte Ausgabe von Matrizen auf Streams testen
// ---------------------------------------------------------------------------
void Matrix4()
{
    cout << "Matrix4()...\n";

    Matrix m(2);
    m.Init(MAT_INIT_IDENTITY);

    // Erst einmal eine "normale" Zahl formatiert ausgeben...
    stringstream strm;
    strm.width(8); strm.precision(4);
    strm << 1.2345678;
    assert(strm.str() == "   1.235");

    // Nun eine Matrix im "Standardformat"...
    strm.str("");
    strm << m;
    assert(strm.str() == "    1.00;     0.00\n    0.00;     1.00\n");

    // Nun eine Matrix in benutzerdefiniertem Format...
    strm.str("");
    strm << MatrixFormat(4, 2, false, false, ":") << m;
    assert(strm.str() == "1.00:0.00:0.00:1.00");

    // Testen, ob das urspruengliche Format des Streams noch intakt ist...
    strm.str("");
    strm << 1.2345678;
    assert(strm.str() == "1.235");

    // Eine Matrix im MATLAB-Format einlesen...
    Matrix m1 = MATLABMatrix( "[ 1, 2; 3.5  4 ]");
    Matrix m2 = MATLABMatrix(L"[ 1  2; 3.5,  4]");
    assert(m1.Dimension() == 2);
    assert(m2.Dimension() == 2);
    assert(is_equal(m1(0, 0), m2(0, 0)));
    assert(is_equal(m1(0, 1), m2(0, 1)));
    assert(is_equal(m1(1, 0), m2(1, 0)));
    assert(is_equal(m1(1, 1), m2(1, 1)));

    Matrix m3 = MATLABMatrix("[-2.2]");
    assert(m3.Dimension() == 1);
    assert(is_equal(m3(0, 0), -2.2));

    // Bei Einlesefehlern werden Ausnahmen ausgeloest...
    try { MATLABMatrix("[1 2; 3 4; 5 6]"); assert(!"a"); } catch(...) { }
    try { MATLABMatrix("[1; 2]"         ); assert(!"b"); } catch(...) { }
    try { MATLABMatrix("[]"             ); assert(!"c"); } catch(...) { }
    try { MATLABMatrix("[1 2; 3 4 5]"   ); assert(!"d"); } catch(...) { }
    try { MATLABMatrix("[1 2]"          ); assert(!"e"); } catch(...) { }
}

// ---------------------------------------------------------------------------
// Speichern und Laden von Matrizen in Dateien testen
// ---------------------------------------------------------------------------
void Matrix5()
{
    cout << "Matrix5()...\n";

    // Wie generiert man (portabel!) temporaere Dateien oder
    // Dateinamen unter Linux und Windows (Vista!)...?!
    const char* pFilename = "_XtestX_.csv";

    Matrix m(10), n(10);
    m.Init(MAT_INIT_HILBERT);

    WriteCSV(m, pFilename);
    ReadCSV(n, pFilename);
    assert(is_equal(m, n));
    remove(pFilename);
}

// ---------------------------------------------------------------------------
// Berechnungen mit Polynomen
// ---------------------------------------------------------------------------
void Matrix6()
{
    cout << "Matrix6()...\n";

    // Polynom 6x^4 - x^3 + 4x^2 + 2x +2 bei x=0 und x=-5.5 ausrechnen
    Vector p1(5);
    p1.Elements() = 6, -1, 4, 2, 2;
    assert(is_equal(p1.PolyVal( 0.0),     2.0));
    assert(is_equal(p1.PolyVal(-5.5), 5768.75));

    // Charakteristisches Polynom berechnen, Beispielwerte stammen von:
    // http://www.arndt-bruenner.de/mathe/scripts/eigenwert2.htm
    Matrix m1(4);
    m1.Row(0) =     38.0,  -43.0/7.0, -63.0/2.0, -1149.0/14.0;
    m1.Row(1) =     14.0,  -19.0/7.0,      -7.0,   -181.0/7.0; 
    m1.Row(2) =  8.0/7.0, 122.0/49.0, -24.0/7.0,  -177.0/49.0;
    m1.Row(3) =     16.0,  -26.0/7.0,     -13.0,   -244.0/7.0;

    Vector p2 = m1.CharPol();
    assert(is_equal(p2(0), 1.0));
    assert(is_equal(p2(1), 3.0));
    assert(is_equal(p2(2), -25.0));
    assert(is_equal(p2(3), -75.0));

    // Nullstellen des charakteristischen Polynoms berechnen (= Eigenwerte)
    DComplexVector p3 = p2.Roots();
    //cout << p3[0] << "\n";   // (-3,1.54074e-033)
    //cout << p3[1] << "\n";   // (5,0)
    //cout << p3[2] << "\n";   // (-5,-8.60958e-042)
    //cout << p3[3] << "\n";   // (-1.21266e-013,-1.75162e-046)
    assert(is_equal(p3[0].real(), -3)); assert(is_equal(p3[0].imag(), 0));
    assert(is_equal(p3[1].real(),  5)); assert(is_equal(p3[1].imag(), 0));
    assert(is_equal(p3[2].real(), -5)); assert(is_equal(p3[2].imag(), 0));
    assert(is_equal(p3[3].real(),  0)); assert(is_equal(p3[3].imag(), 0));
}

// ---------------------------------------------------------------------------
// Eigenwerte
// ---------------------------------------------------------------------------
void Matrix7()
{
    cout << "Matrix7()...\n";
        
    // Eigenwerte einer (symmetrischen) Hilbertmatrix, Jacobi-Verfahren
    Matrix a(50), eig(50), vec(50), test(50);
    a.Init(MAT_INIT_HILBERT);
    a.Jacobi(eig, vec);
    test = a * vec - vec * eig;
    assert(is_equal(test.GetMax(), 0));
    assert(is_equal(test.GetMin(), 0));

    // Eng benachbarte Eigenwerte, kleine Diagonalmatrix, Jacobi-Verfahren
    Matrix a1(2), eig1(2), vec1(2), test1(2);
    a1.Row(0) = 0.999, 0;
    a1.Row(1) =     0, 1;
    a1.Jacobi(eig1, vec1);
    test1 = a1 * vec1 - vec1 * eig1;
    assert(is_equal(test1.GetMax(), 0));
    assert(is_equal(test1.GetMin(), 0));

    // Eng benachbarte Eigenwerte, kleine Diagonalmatrix, von-Mises-Iteration
    double eigw1;
    a1.Row(0) = 0.999, 0;
    a1.Row(1) =     0, 1;
    Vector v1 = a1.Mises(eigw1);
    assert(eigw1 > 0.9999 && eigw1 < 1.0001);
    assert(is_equal(v1(0), 0));
    assert(is_equal(v1(1) * v1(1), 1));

    // Ein reeller und zwei komplexe Eigenwerte, von-Mieses-Iteration
    Matrix a2(3);
    double eigw2;
    a2.Row(0) =   0, .5,  0;
    a2.Row(1) = -.5,  0,  0;
    a2.Row(2) =   0,  0, -1;
    Vector v2 = a2.Mises(eigw2);
    assert(is_equal(v2(0),  0));
    assert(is_equal(v2(1),  0));
    assert(is_equal(v2(2) * v2(2), 1));
    assert(is_equal(eigw2, -1));

    // Abbruch mit Exception bei fehlender Konvergenz, von-Mises-Verfahren
    try
    {
        double dummy;
        a1.Row(0) =  0, 1;
        a1.Row(1) = -1, 0;
        a1.Mises(dummy);
    }
    catch(const std::runtime_error& /* err */)
    {
        // string msg = err.what();
        // std::cout << msg << std::endl;
        return;
    }
    assert(!"Matrix7(): Exception wurde nicht ausgeloest!");
}

// ---------------------------------------------------------------------------
// Lineare Gleichungssysteme
// ---------------------------------------------------------------------------
void Matrix8()
{
    // Erst eine kleine, einfache Matrix testen...
    cout << "Matrix8()...\n";
        
    Matrix a1(4);
    a1.Row(0) =  0,  0, -3, -8;
    a1.Row(1) = -8, -2,  0, 12;
    a1.Row(2) =  4,  0,  0, 12;
    a1.Row(3) =  0,  0, -3, -4;

    Vector y1(4);
    y1.Elements() = -28, 0, -2, 0;

    Vector x1 = LinSolve(a1, y1);
    assert(is_equal(x1[0], -21.5000000));
    assert(is_equal(x1[1], 128.0000000));
    assert(is_equal(x1[2],  -9.3333333));
    assert(is_equal(x1[3],   7.0000000));

    // Jetzt mit einer grossen Hilbert-Matrix...
    for(size_t dim = 90; dim < (size_t)100; ++dim)
    {
        Matrix a2(dim), q2(dim), r2(dim);
        a2.Init(MAT_INIT_HILBERT);
        a2.QRDecomp(q2, r2);

        Vector y2(dim);
        y2.Fill(1.0);

        Vector x2 = LinSolveQR(q2, r2, y2);
        Vector delta = a2 * x2 - y2;

        assert(is_equal(delta.GetMax(), 0));
        assert(is_equal(delta.GetMin(), 0));
    }
}

// ---------------------------------------------------------------------------
// Hauptprogramm
// ---------------------------------------------------------------------------
int main()
{
#ifdef _MSC_VER
    // Suche nach Memory-Leaks bei Microsoft Visual C++ aktivieren
    _CrtSetDbgFlag ( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );
#endif

    Vector1();  // Initialisierung von bzw. Zugriff auf Vektoren testen
    Vector2();  // Grundrechenarten von Vektoren testen
    Vector3();  // Weitere Funktionalitaeten von Vektoren testen
    Vector4();  // Formatierte Ausgabe von Vektoren auf Streams testen
    Vector5();  // Fast Fourier Transformation
    Matrix1();  // Initialisierung von bzw. Zugriff auf Matrizen testen
    Matrix2();  // Berechnungen mit Matrizen testen
    Matrix3();  // Weitere Funktionalitaeten von Matrizen testen
    Matrix4();  // Formatierte Ausgabe von Matrizen auf Streams testen
    Matrix5();  // Speichern und Laden von Matrizen in Dateien testen
    Matrix6();  // Berechnungen mit Polynomen
    Matrix7();  // Eigenwerte
    Matrix8();  // Lineare Gleichungssysteme
}
